// Copyright 2022 schukai GmbH
// SPDX-License-Identifier: AGPL-3.0

package xflags

import (
	"github.com/stretchr/testify/assert"
	"testing"
)

type testInterfaceCallbacks func(s *Settings[testIntegrationStruct])

type testIntegrationStruct struct {
	Help    bool `short:"h" long:"help" description:"Show this help message"`
	Verbose bool `short:"v" long:"verbose" description:"Enable verbose logging"`
}

type testIntegrationTestCases[C any] struct {
	name   string
	args   []string
	test   testInterfaceCallbacks
	errors []string
}

func TestIntegrationError(t *testing.T) {

	tests := []testIntegrationTestCases[testIntegrationStruct]{
		{
			name: "test",
			args: []string{"-a"},
			errors: []string{
				"flag provided but not defined: -a",
			},
		},
	}

	for _, tt := range tests {
		t.Run(tt.args[0], func(t *testing.T) {
			settings := New(tt.name, testIntegrationStruct{})
			assert.NotNil(t, settings)

			s := settings.Parse(tt.args)
			if !s.HasErrors() {
				t.Error(s.Errors())
			}

			summarizedErrors := []string{}
			for _, e := range s.Errors() {
				summarizedErrors = append(summarizedErrors, e.Error())
			}

			for _, err := range tt.errors {
				assert.Contains(t, summarizedErrors, err)
			}

		})
	}

}

func TestIntegration(t *testing.T) {

	tests := []testIntegrationTestCases[testIntegrationStruct]{
		{
			name: "test",
			args: []string{},
			test: func(s *Settings[testIntegrationStruct]) {
				v := s.GetValues()
				assert.NotNil(t, v)
				assert.IsType(t, testIntegrationStruct{}, v)
				assert.False(t, v.Verbose)
				assert.False(t, s.GetValues().Verbose)
			},
		},
		{
			name: "test",
			args: []string{"-v"},
			test: func(s *Settings[testIntegrationStruct]) {
				v := s.GetValues()
				assert.NotNil(t, v)
				assert.IsType(t, testIntegrationStruct{}, v)
				assert.True(t, v.Verbose)
				assert.True(t, s.GetValues().Verbose)
			},
		},
		{
			name: "test",
			args: []string{"--verbose"},
			test: func(s *Settings[testIntegrationStruct]) {
				assert.True(t, s.GetValues().Verbose)
			},
		},
		{
			name: "test",
			args: []string{"-verbose"},
			test: func(s *Settings[testIntegrationStruct]) {
				assert.True(t, s.GetValues().Verbose)
			},
		},
		{
			name: "test",
			args: []string{"sub1"},
			test: func(s *Settings[testIntegrationStruct]) {
				assert.False(t, s.GetValues().Verbose)
			},
		},
	}

	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			settings := New(tt.name, testIntegrationStruct{})
			assert.NotNil(t, settings)

			s := settings.Parse(tt.args)

			if s.HasErrors() {
				t.Error(s.Errors())
			}

			assert.Equal(t, settings, s)

			tt.test(settings)

		})
	}
}
