// Copyright 2022 schukai GmbH
// SPDX-License-Identifier: AGPL-3.0

package xflags

import (
	"strconv"
	"strings"
	"testing"

	"github.com/stretchr/testify/assert"
)

type testCmdStructIssue1 struct {
	Cmd1 struct {
		Cmd2 struct {
			Cmd3 struct {
				V3 bool `short:"v3"`
			} `command:"cmd3" call:"DoCmd3"`
		} `command:"cmd2"`
		Cmd4 struct {
			Cmd5 struct {
			} `command:"cmd5" call:"DoCmd5"`
		} `command:"cmd4"`
		Cmd6 struct {
			Cmd7 struct {
			} `command:"cmd7"`
		} `command:"cmd6"`
	} `command:"cmd1"`
}

func (c *testCmdStructIssue1) DoCmd3(s *Settings[testCmdStructIssue1]) {

}
func (c *testCmdStructIssue1) DoCmd5(s *Settings[testCmdStructIssue1]) {

}

type testStructIssue1Config struct {
}

// here it is tested whether the last and defined callback is called
func TestIssue1TestCallCMD4(tp *testing.T) {

	testData := []struct {
		args       []string
		isExecuted bool
		hasErrors  bool
		hasHint    bool
	}{
		{[]string{"cmd1"}, false, true, true},
		{[]string{}, false, true, true},
		{[]string{"cmd1", "cmd2"}, false, true, true},
		{[]string{"cmd1", "cmd2", "cmd3"}, true, false, false},
		{[]string{"cmd1", "cmd4", "cmd5"}, true, false, false},
		{[]string{"cmd1"}, false, true, true},
		{[]string{"cmd1", "cmd6"}, false, true, true},
		{[]string{"cmd1", "cmd6", "cmd7"}, false, true, false},
	}

	for i, tt := range testData {
		tp.Run(strconv.Itoa(i)+":"+strings.Join(tt.args, ","), func(t *testing.T) {
			s := execute(testCmdStructIssue1{}, noShadow{}, "test", tt.args)
			assert.Equal(t, tt.hasErrors, s.HasErrors())
			assert.Equal(t, tt.hasHint, s.HasHint())

			if !tt.hasErrors && s.HasErrors() {
				t.Log(s.Errors())
			}

			assert.Equal(t, tt.isExecuted, s.WasExecuted())
		})
	}

}

func TestIssue1TestNoCallback(t *testing.T) {
	s := execute(testCmdStructIssue1{}, noShadow{}, "test", []string{"cmd1", "cmd6", "cmd7"})
	assert.Equal(t, 3, len(s.Errors()))
	assert.False(t, s.WasExecuted())
}

func TestIssue1TestToMuchCommands(t *testing.T) {
	s := execute(testCmdStructIssue1{}, noShadow{}, "test", []string{"cmd1", "cmd9"})
	assert.True(t, s.MissingCommand())
	assert.False(t, s.WasExecuted())
}

// here it is tested whether the last and defined callback is called
func TestIssue1TestCallCMD3(t *testing.T) {
	s := execute(testCmdStructIssue1{}, noShadow{}, "test", []string{"cmd1", "cmd2", "cmd3"})
	assert.Equal(t, 0, len(s.Errors()))
	assert.True(t, s.WasExecuted())
}

// NoShadow is an internal Struct for testing
func TestIssue1MessageWithNoShadow(t *testing.T) {
	s := execute(testCmdStructIssue1{}, noShadow{}, "test", []string{"cmd1", "cmd2", "cmd3", "-v3"})
	assert.Equal(t, 0, len(s.Errors()))
}

func TestIssue1MRequestHelp(t *testing.T) {

	s := New("test", testCmdStructIssue1{})
	s.Parse([]string{"cmd1", "cmd2", "--help"})
	assert.True(t, s.HelpRequested())
}

func TestIssue1A(t *testing.T) {

	s := New("test", testCmdStructIssue1{})
	s.Parse([]string{"cmd1", "cmd2", "cmd3", "-v3"})
	assert.False(t, s.HelpRequested())

	assert.Equal(t, 0, len(s.Errors()))
	assert.True(t, s.GetValues().Cmd1.Cmd2.Cmd3.V3)
}
func TestIssue1HelpRequested(t *testing.T) {

	s := New("test", testCmdStructIssue1{})
	s.Parse([]string{"cmd1", "-h"})
	assert.True(t, s.HelpRequested())

	assert.Equal(t, 1, len(s.Errors()))
}

func TestIssue1Summary(tp *testing.T) {

	data := []struct {
		args           []string
		errorCount     int
		helpRequested  bool
		wasExecuted    bool
		missingCommand bool
	}{
		{[]string{"cmd1", "cmd2", "cmd3", "-v3"}, 0, false, true, false},
	}

	for _, tt := range data {
		tp.Run(tt.args[0], func(t *testing.T) {

			s := execute(testCmdStructIssue1{}, noShadow{}, "test", tt.args)
			assert.Equal(t, tt.helpRequested, s.HelpRequested())
			assert.Equal(t, tt.wasExecuted, s.WasExecuted())
			assert.Equal(t, tt.missingCommand, s.MissingCommand())
			assert.Equal(t, tt.errorCount, len(s.Errors()))
		})
	}

}

func TestIssue1B(t *testing.T) {

	s := New("test", testCmdStructIssue1{})
	s.Parse([]string{"cmd1", "cmd2", "cmd3"})
	assert.False(t, s.HelpRequested())

	assert.Equal(t, 0, len(s.Errors()))
	assert.False(t, s.GetValues().Cmd1.Cmd2.Cmd3.V3)
}
